from typing import List, Literal
from enum import Enum
from pyVmomi import vim, vmodl
from datetime import datetime
from pyVmomi.VmomiSupport import ManagedObject, NoneType


class CryptoManager(ManagedObject):
    @property
    def enabled(self) -> bool: ...
    def AddKey(self, key: CryptoKeyPlain) -> NoneType: ...
    def AddKeys(self, keys: List[CryptoKeyPlain]) -> List[CryptoKeyResult]: ...
    def RemoveKey(self, key: CryptoKeyId, force: bool) -> NoneType: ...
    def RemoveKeys(self, keys: List[CryptoKeyId], force: bool) -> List[CryptoKeyResult]: ...
    def ListKeys(self, limit: int) -> List[CryptoKeyId]: ...


class CryptoManagerHost(CryptoManager):
    def Prepare(self) -> NoneType: ...
    def Enable(self, initialKey: CryptoKeyPlain) -> NoneType: ...
    def ChangeKey(self, newKey: CryptoKeyPlain) -> vim.Task: ...
    def Disable(self) -> NoneType: ...
    def GetCryptoKeyStatus(self, keys: List[CryptoKeyId]) -> List[CryptoManagerHost.KeyStatus]: ...


    class KeyStatus(vmodl.DynamicData):
        @property
        def keyId(self) -> CryptoKeyId: ...
        @keyId.setter
        def keyId(self, value: CryptoKeyId):
            self._keyId = value
        @property
        def present(self) -> bool: ...
        @present.setter
        def present(self, value: bool):
            self._present = value
        @property
        def managementType(self) -> str: ...
        @managementType.setter
        def managementType(self, value: str):
            self._managementType = value


    class KeyManagementType(Enum):
        unknown = "unknown"
        internal = "internal"
        external = "external"


class CryptoManagerHostKMS(CryptoManagerHost): ...


class CryptoManagerKmip(CryptoManager):
    @property
    def kmipServers(self) -> List[KmipClusterInfo]: ...
    def RegisterKmipServer(self, server: KmipServerSpec) -> NoneType: ...
    def MarkDefault(self, clusterId: KeyProviderId) -> NoneType: ...
    def UpdateKmipServer(self, server: KmipServerSpec) -> NoneType: ...
    def RemoveKmipServer(self, clusterId: KeyProviderId, serverName: str) -> NoneType: ...
    def ListKmipServers(self, limit: int) -> List[KmipClusterInfo]: ...
    def RetrieveKmipServersStatus(self, clusters: List[KmipClusterInfo]) -> vim.Task: ...
    def GenerateKey(self, keyProvider: KeyProviderId, spec: CryptoManagerKmip.CustomAttributeSpec) -> CryptoKeyResult: ...
    def RetrieveKmipServerCert(self, keyProvider: KeyProviderId, server: KmipServerInfo) -> CryptoManagerKmip.ServerCertInfo: ...
    def UploadKmipServerCert(self, cluster: KeyProviderId, certificate: str) -> NoneType: ...
    def GenerateSelfSignedClientCert(self, cluster: KeyProviderId, request: CryptoManagerKmip.CertSignRequest) -> str: ...
    def GenerateClientCsr(self, cluster: KeyProviderId, request: CryptoManagerKmip.CertSignRequest) -> str: ...
    def RetrieveSelfSignedClientCert(self, cluster: KeyProviderId) -> str: ...
    def RetrieveClientCsr(self, cluster: KeyProviderId) -> str: ...
    def RetrieveClientCert(self, cluster: KeyProviderId) -> str: ...
    def UpdateSelfSignedClientCert(self, cluster: KeyProviderId, certificate: str) -> NoneType: ...
    def UpdateKmsSignedCsrClientCert(self, cluster: KeyProviderId, certificate: str) -> NoneType: ...
    def UploadClientCert(self, cluster: KeyProviderId, certificate: str, privateKey: str) -> NoneType: ...
    def IsKmsClusterActive(self, cluster: KeyProviderId) -> bool: ...
    def SetDefaultKmsCluster(self, entity: vim.ManagedEntity, clusterId: KeyProviderId) -> NoneType: ...
    def GetDefaultKmsCluster(self, entity: vim.ManagedEntity, defaultsToParent: bool) -> KeyProviderId: ...
    def QueryCryptoKeyStatus(self, keyIds: List[CryptoKeyId], checkKeyBitMap: int) -> List[CryptoManagerKmip.CryptoKeyStatus]: ...
    def RegisterKmsCluster(self, clusterId: KeyProviderId, managementType: str) -> NoneType: ...
    def UnregisterKmsCluster(self, clusterId: KeyProviderId) -> NoneType: ...
    def ListKmsClusters(self, includeKmsServers: bool, managementTypeFilter: int, statusFilter: int) -> List[KmipClusterInfo]: ...
    def SetKeyCustomAttributes(self, keyId: CryptoKeyId, spec: CryptoManagerKmip.CustomAttributeSpec) -> CryptoKeyResult: ...


    class CertSignRequest(vmodl.DynamicData):
        @property
        def commonName(self) -> str: ...
        @commonName.setter
        def commonName(self, value: str):
            self._commonName = value
        @property
        def organization(self) -> str: ...
        @organization.setter
        def organization(self, value: str):
            self._organization = value
        @property
        def organizationUnit(self) -> str: ...
        @organizationUnit.setter
        def organizationUnit(self, value: str):
            self._organizationUnit = value
        @property
        def locality(self) -> str: ...
        @locality.setter
        def locality(self, value: str):
            self._locality = value
        @property
        def state(self) -> str: ...
        @state.setter
        def state(self, value: str):
            self._state = value
        @property
        def country(self) -> str: ...
        @country.setter
        def country(self, value: str):
            self._country = value
        @property
        def email(self) -> str: ...
        @email.setter
        def email(self, value: str):
            self._email = value


    class CertificateInfo(vmodl.DynamicData):
        @property
        def subject(self) -> str: ...
        @subject.setter
        def subject(self, value: str):
            self._subject = value
        @property
        def issuer(self) -> str: ...
        @issuer.setter
        def issuer(self, value: str):
            self._issuer = value
        @property
        def serialNumber(self) -> str: ...
        @serialNumber.setter
        def serialNumber(self, value: str):
            self._serialNumber = value
        @property
        def notBefore(self) -> datetime: ...
        @notBefore.setter
        def notBefore(self, value: datetime):
            self._notBefore = value
        @property
        def notAfter(self) -> datetime: ...
        @notAfter.setter
        def notAfter(self, value: datetime):
            self._notAfter = value
        @property
        def fingerprint(self) -> str: ...
        @fingerprint.setter
        def fingerprint(self, value: str):
            self._fingerprint = value
        @property
        def checkTime(self) -> datetime: ...
        @checkTime.setter
        def checkTime(self, value: datetime):
            self._checkTime = value
        @property
        def secondsSinceValid(self) -> int: ...
        @secondsSinceValid.setter
        def secondsSinceValid(self, value: int):
            self._secondsSinceValid = value
        @property
        def secondsBeforeExpire(self) -> int: ...
        @secondsBeforeExpire.setter
        def secondsBeforeExpire(self, value: int):
            self._secondsBeforeExpire = value


    class ClusterStatus(vmodl.DynamicData):
        @property
        def clusterId(self) -> KeyProviderId: ...
        @clusterId.setter
        def clusterId(self, value: KeyProviderId):
            self._clusterId = value
        @property
        def overallStatus(self) -> vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']: ...
        @overallStatus.setter
        def overallStatus(self, value: vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']):
            self._overallStatus = value
        @property
        def managementType(self) -> str: ...
        @managementType.setter
        def managementType(self, value: str):
            self._managementType = value
        @property
        def servers(self) -> List[CryptoManagerKmip.ServerStatus]: ...
        @servers.setter
        def servers(self, value: List[CryptoManagerKmip.ServerStatus]):
            self._servers = value
        @property
        def clientCertInfo(self) -> CryptoManagerKmip.CertificateInfo: ...
        @clientCertInfo.setter
        def clientCertInfo(self, value: CryptoManagerKmip.CertificateInfo):
            self._clientCertInfo = value


    class CryptoKeyStatus(vmodl.DynamicData):
        @property
        def keyId(self) -> CryptoKeyId: ...
        @keyId.setter
        def keyId(self, value: CryptoKeyId):
            self._keyId = value
        @property
        def keyAvailable(self) -> bool: ...
        @keyAvailable.setter
        def keyAvailable(self, value: bool):
            self._keyAvailable = value
        @property
        def reason(self) -> str: ...
        @reason.setter
        def reason(self, value: str):
            self._reason = value
        @property
        def encryptedVMs(self) -> List[vim.VirtualMachine]: ...
        @encryptedVMs.setter
        def encryptedVMs(self, value: List[vim.VirtualMachine]):
            self._encryptedVMs = value
        @property
        def affectedHosts(self) -> List[vim.HostSystem]: ...
        @affectedHosts.setter
        def affectedHosts(self, value: List[vim.HostSystem]):
            self._affectedHosts = value
        @property
        def referencedByTags(self) -> List[str]: ...
        @referencedByTags.setter
        def referencedByTags(self, value: List[str]):
            self._referencedByTags = value


        class KeyUnavailableReason(Enum):
            KeyStateMissingInCache = "KeyStateMissingInCache"
            KeyStateClusterInvalid = "KeyStateClusterInvalid"
            KeyStateClusterUnreachable = "KeyStateClusterUnreachable"
            KeyStateMissingInKMS = "KeyStateMissingInKMS"
            KeyStateNotActiveOrEnabled = "KeyStateNotActiveOrEnabled"
            KeyStateManagedByTrustAuthority = "KeyStateManagedByTrustAuthority"
            KeyStateManagedByNKP = "KeyStateManagedByNKP"


    class CustomAttributeSpec(vmodl.DynamicData):
        @property
        def attributes(self) -> List[vim.KeyValue]: ...
        @attributes.setter
        def attributes(self, value: List[vim.KeyValue]):
            self._attributes = value


    class ServerCertInfo(vmodl.DynamicData):
        @property
        def certificate(self) -> str: ...
        @certificate.setter
        def certificate(self, value: str):
            self._certificate = value
        @property
        def certInfo(self) -> CryptoManagerKmip.CertificateInfo: ...
        @certInfo.setter
        def certInfo(self, value: CryptoManagerKmip.CertificateInfo):
            self._certInfo = value
        @property
        def clientTrustServer(self) -> bool: ...
        @clientTrustServer.setter
        def clientTrustServer(self, value: bool):
            self._clientTrustServer = value


    class ServerStatus(vmodl.DynamicData):
        @property
        def name(self) -> str: ...
        @name.setter
        def name(self, value: str):
            self._name = value
        @property
        def status(self) -> vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']: ...
        @status.setter
        def status(self, value: vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']):
            self._status = value
        @property
        def connectionStatus(self) -> str: ...
        @connectionStatus.setter
        def connectionStatus(self, value: str):
            self._connectionStatus = value
        @property
        def certInfo(self) -> CryptoManagerKmip.CertificateInfo: ...
        @certInfo.setter
        def certInfo(self, value: CryptoManagerKmip.CertificateInfo):
            self._certInfo = value
        @property
        def clientTrustServer(self) -> bool: ...
        @clientTrustServer.setter
        def clientTrustServer(self, value: bool):
            self._clientTrustServer = value
        @property
        def serverTrustClient(self) -> bool: ...
        @serverTrustClient.setter
        def serverTrustClient(self, value: bool):
            self._serverTrustClient = value


class CryptoKeyId(vmodl.DynamicData):
    @property
    def keyId(self) -> str: ...
    @keyId.setter
    def keyId(self, value: str):
        self._keyId = value
    @property
    def providerId(self) -> KeyProviderId: ...
    @providerId.setter
    def providerId(self, value: KeyProviderId):
        self._providerId = value


class CryptoKeyPlain(vmodl.DynamicData):
    @property
    def keyId(self) -> CryptoKeyId: ...
    @keyId.setter
    def keyId(self, value: CryptoKeyId):
        self._keyId = value
    @property
    def algorithm(self) -> str: ...
    @algorithm.setter
    def algorithm(self, value: str):
        self._algorithm = value
    @property
    def keyData(self) -> str: ...
    @keyData.setter
    def keyData(self, value: str):
        self._keyData = value


class CryptoKeyResult(vmodl.DynamicData):
    @property
    def keyId(self) -> CryptoKeyId: ...
    @keyId.setter
    def keyId(self, value: CryptoKeyId):
        self._keyId = value
    @property
    def success(self) -> bool: ...
    @success.setter
    def success(self, value: bool):
        self._success = value
    @property
    def reason(self) -> str: ...
    @reason.setter
    def reason(self, value: str):
        self._reason = value
    @property
    def fault(self) -> vmodl.MethodFault: ...
    @fault.setter
    def fault(self, value: vmodl.MethodFault):
        self._fault = value


class CryptoSpec(vmodl.DynamicData): ...


class CryptoSpecDecrypt(CryptoSpec): ...


class CryptoSpecDeepRecrypt(CryptoSpec):
    @property
    def newKeyId(self) -> CryptoKeyId: ...
    @newKeyId.setter
    def newKeyId(self, value: CryptoKeyId):
        self._newKeyId = value


class CryptoSpecEncrypt(CryptoSpec):
    @property
    def cryptoKeyId(self) -> CryptoKeyId: ...
    @cryptoKeyId.setter
    def cryptoKeyId(self, value: CryptoKeyId):
        self._cryptoKeyId = value


class CryptoSpecNoOp(CryptoSpec): ...


class CryptoSpecRegister(CryptoSpecNoOp):
    @property
    def cryptoKeyId(self) -> CryptoKeyId: ...
    @cryptoKeyId.setter
    def cryptoKeyId(self, value: CryptoKeyId):
        self._cryptoKeyId = value


class CryptoSpecShallowRecrypt(CryptoSpec):
    @property
    def newKeyId(self) -> CryptoKeyId: ...
    @newKeyId.setter
    def newKeyId(self, value: CryptoKeyId):
        self._newKeyId = value


class KeyProviderId(vmodl.DynamicData):
    @property
    def id(self) -> str: ...
    @id.setter
    def id(self, value: str):
        self._id = value


class KmipClusterInfo(vmodl.DynamicData):
    @property
    def clusterId(self) -> KeyProviderId: ...
    @clusterId.setter
    def clusterId(self, value: KeyProviderId):
        self._clusterId = value
    @property
    def servers(self) -> List[KmipServerInfo]: ...
    @servers.setter
    def servers(self, value: List[KmipServerInfo]):
        self._servers = value
    @property
    def useAsDefault(self) -> bool: ...
    @useAsDefault.setter
    def useAsDefault(self, value: bool):
        self._useAsDefault = value
    @property
    def managementType(self) -> str: ...
    @managementType.setter
    def managementType(self, value: str):
        self._managementType = value
    @property
    def useAsEntityDefault(self) -> List[vim.ManagedEntity]: ...
    @useAsEntityDefault.setter
    def useAsEntityDefault(self, value: List[vim.ManagedEntity]):
        self._useAsEntityDefault = value
    @property
    def hasBackup(self) -> bool: ...
    @hasBackup.setter
    def hasBackup(self, value: bool):
        self._hasBackup = value
    @property
    def tpmRequired(self) -> bool: ...
    @tpmRequired.setter
    def tpmRequired(self, value: bool):
        self._tpmRequired = value
    @property
    def keyId(self) -> str: ...
    @keyId.setter
    def keyId(self, value: str):
        self._keyId = value


    class KmsManagementType(Enum):
        unknown = "unknown"
        vCenter = "vCenter"
        trustAuthority = "trustAuthority"
        nativeProvider = "nativeProvider"


class KmipServerInfo(vmodl.DynamicData):
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str):
        self._name = value
    @property
    def address(self) -> str: ...
    @address.setter
    def address(self, value: str):
        self._address = value
    @property
    def port(self) -> int: ...
    @port.setter
    def port(self, value: int):
        self._port = value
    @property
    def proxyAddress(self) -> str: ...
    @proxyAddress.setter
    def proxyAddress(self, value: str):
        self._proxyAddress = value
    @property
    def proxyPort(self) -> int: ...
    @proxyPort.setter
    def proxyPort(self, value: int):
        self._proxyPort = value
    @property
    def reconnect(self) -> int: ...
    @reconnect.setter
    def reconnect(self, value: int):
        self._reconnect = value
    @property
    def protocol(self) -> str: ...
    @protocol.setter
    def protocol(self, value: str):
        self._protocol = value
    @property
    def nbio(self) -> int: ...
    @nbio.setter
    def nbio(self, value: int):
        self._nbio = value
    @property
    def timeout(self) -> int: ...
    @timeout.setter
    def timeout(self, value: int):
        self._timeout = value
    @property
    def userName(self) -> str: ...
    @userName.setter
    def userName(self, value: str):
        self._userName = value


class KmipServerSpec(vmodl.DynamicData):
    @property
    def clusterId(self) -> KeyProviderId: ...
    @clusterId.setter
    def clusterId(self, value: KeyProviderId):
        self._clusterId = value
    @property
    def info(self) -> KmipServerInfo: ...
    @info.setter
    def info(self, value: KmipServerInfo):
        self._info = value
    @property
    def password(self) -> str: ...
    @password.setter
    def password(self, value: str):
        self._password = value


class KmipServerStatus(vmodl.DynamicData):
    @property
    def clusterId(self) -> KeyProviderId: ...
    @clusterId.setter
    def clusterId(self, value: KeyProviderId):
        self._clusterId = value
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str):
        self._name = value
    @property
    def status(self) -> vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']: ...
    @status.setter
    def status(self, value: vim.ManagedEntity.Status | Literal['gray', 'green', 'yellow', 'red']):
        self._status = value
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, value: str):
        self._description = value


from typing import List
from enum import Enum
from pyVmomi import vim, vmodl, vslm
from datetime import datetime
from pyVmomi.VmomiSupport import ManagedObject, NoneType, long


class VStorageObjectManager(ManagedObject):
    def CreateDisk(self, spec: vim.vslm.CreateSpec) -> vslm.Task: ...
    def RegisterDisk(self, path: str, name: str) -> vim.vslm.VStorageObject: ...
    def ExtendDisk(self, id: vim.vslm.ID, newCapacityInMB: long) -> vslm.Task: ...
    def InflateDisk(self, id: vim.vslm.ID) -> vslm.Task: ...
    def RenameVStorageObject(self, id: vim.vslm.ID, name: str) -> NoneType: ...
    def UpdateVStorageObjectPolicy(self, id: vim.vslm.ID, profile: List[vim.vm.ProfileSpec]) -> vslm.Task: ...
    def UpdateVStorageObjectCrypto(self, id: vim.vslm.ID, profile: List[vim.vm.ProfileSpec], disksCrypto: vim.vslm.DiskCryptoSpec) -> vslm.Task: ...
    def UpdateVStorageInfrastructureObjectPolicy(self, spec: vim.vslm.InfrastructureObjectPolicySpec) -> vslm.Task: ...
    def RetrieveVStorageInfrastructureObjectPolicy(self, datastore: vim.Datastore) -> List[vim.vslm.InfrastructureObjectPolicy]: ...
    def DeleteVStorageObject(self, id: vim.vslm.ID) -> vslm.Task: ...
    def RetrieveVStorageObject(self, id: vim.vslm.ID) -> vim.vslm.VStorageObject: ...
    def RetrieveVStorageObjectState(self, id: vim.vslm.ID) -> vim.vslm.StateInfo: ...
    def RetrieveVStorageObjectAssociations(self, ids: List[vim.vslm.ID]) -> List[VStorageObjectAssociations]: ...
    def ListVStorageObjectsForSpec(self, query: List[VStorageObjectQuerySpec], maxResult: int) -> VStorageObjectQueryResult: ...
    def CloneVStorageObject(self, id: vim.vslm.ID, spec: vim.vslm.CloneSpec) -> vslm.Task: ...
    def RelocateVStorageObject(self, id: vim.vslm.ID, spec: vim.vslm.RelocateSpec) -> vslm.Task: ...
    def SetVStorageObjectControlFlags(self, id: vim.vslm.ID, controlFlags: List[str]) -> NoneType: ...
    def ClearVStorageObjectControlFlags(self, id: vim.vslm.ID, controlFlags: List[str]) -> NoneType: ...
    def AttachTagToVStorageObject(self, id: vim.vslm.ID, category: str, tag: str) -> NoneType: ...
    def DetachTagFromVStorageObject(self, id: vim.vslm.ID, category: str, tag: str) -> NoneType: ...
    def ListVStorageObjectsAttachedToTag(self, category: str, tag: str) -> List[vim.vslm.ID]: ...
    def ListTagsAttachedToVStorageObject(self, id: vim.vslm.ID) -> List[vim.vslm.TagEntry]: ...
    def ReconcileDatastoreInventory(self, datastore: vim.Datastore) -> vslm.Task: ...
    def ScheduleReconcileDatastoreInventory(self, datastore: vim.Datastore) -> NoneType: ...
    def CreateSnapshot(self, id: vim.vslm.ID, description: str) -> vslm.Task: ...
    def DeleteSnapshot(self, id: vim.vslm.ID, snapshotId: vim.vslm.ID) -> vslm.Task: ...
    def RetrieveSnapshotInfo(self, id: vim.vslm.ID) -> vim.vslm.VStorageObjectSnapshotInfo: ...
    def CreateDiskFromSnapshot(self, id: vim.vslm.ID, snapshotId: vim.vslm.ID, name: str, profile: List[vim.vm.ProfileSpec], crypto: vim.encryption.CryptoSpec, path: str) -> vslm.Task: ...
    def RevertVStorageObject(self, id: vim.vslm.ID, snapshotId: vim.vslm.ID) -> vslm.Task: ...
    def RetrieveSnapshotDetails(self, id: vim.vslm.ID, snapshotId: vim.vslm.ID) -> vim.vslm.VStorageObjectSnapshotDetails: ...
    def QueryChangedDiskAreas(self, id: vim.vslm.ID, snapshotId: vim.vslm.ID, startOffset: long, changeId: str) -> vim.VirtualMachine.DiskChangeInfo: ...
    def QueryGlobalCatalogSyncStatus(self) -> List[DatastoreSyncStatus]: ...
    def QueryGlobalCatalogSyncStatusForDatastore(self, datastoreURL: str) -> DatastoreSyncStatus: ...
    def UpdateVStorageObjectMetadata(self, id: vim.vslm.ID, metadata: List[vim.KeyValue], deleteKeys: List[str]) -> vslm.Task: ...
    def RetrieveVStorageObjectMetadata(self, id: vim.vslm.ID, snapshotId: vim.vslm.ID, prefix: str) -> List[vim.KeyValue]: ...
    def RetrieveVStorageObjectMetadataValue(self, id: vim.vslm.ID, snapshotId: vim.vslm.ID, key: str) -> str: ...
    def RetrieveVStorageObjects(self, ids: List[vim.vslm.ID]) -> List[VStorageObjectResult]: ...
    def AttachDisk(self, id: vim.vslm.ID, vm: vim.VirtualMachine, controllerKey: int, unitNumber: int) -> vslm.Task: ...


class DatastoreSyncStatus(vmodl.DynamicData):
    @property
    def datastoreURL(self) -> str: ...
    @datastoreURL.setter
    def datastoreURL(self, value: str):
        self._datastoreURL = value
    @property
    def objectVClock(self) -> long: ...
    @objectVClock.setter
    def objectVClock(self, value: long):
        self._objectVClock = value
    @property
    def syncVClock(self) -> long: ...
    @syncVClock.setter
    def syncVClock(self, value: long):
        self._syncVClock = value
    @property
    def syncTime(self) -> datetime: ...
    @syncTime.setter
    def syncTime(self, value: datetime):
        self._syncTime = value
    @property
    def numberOfRetries(self) -> int: ...
    @numberOfRetries.setter
    def numberOfRetries(self, value: int):
        self._numberOfRetries = value
    @property
    def error(self) -> vmodl.MethodFault: ...
    @error.setter
    def error(self, value: vmodl.MethodFault):
        self._error = value


class VStorageObjectAssociations(vmodl.DynamicData):
    @property
    def id(self) -> vim.vslm.ID: ...
    @id.setter
    def id(self, value: vim.vslm.ID):
        self._id = value
    @property
    def vmDiskAssociation(self) -> List[VStorageObjectAssociations.VmDiskAssociation]: ...
    @vmDiskAssociation.setter
    def vmDiskAssociation(self, value: List[VStorageObjectAssociations.VmDiskAssociation]):
        self._vmDiskAssociation = value
    @property
    def fault(self) -> vmodl.MethodFault: ...
    @fault.setter
    def fault(self, value: vmodl.MethodFault):
        self._fault = value


    class VmDiskAssociation(vmodl.DynamicData):
        @property
        def vmId(self) -> str: ...
        @vmId.setter
        def vmId(self, value: str):
            self._vmId = value
        @property
        def diskKey(self) -> int: ...
        @diskKey.setter
        def diskKey(self, value: int):
            self._diskKey = value


class VStorageObjectQueryResult(vmodl.DynamicData):
    @property
    def allRecordsReturned(self) -> bool: ...
    @allRecordsReturned.setter
    def allRecordsReturned(self, value: bool):
        self._allRecordsReturned = value
    @property
    def id(self) -> List[vim.vslm.ID]: ...
    @id.setter
    def id(self, value: List[vim.vslm.ID]):
        self._id = value
    @property
    def queryResults(self) -> List[VStorageObjectResult]: ...
    @queryResults.setter
    def queryResults(self, value: List[VStorageObjectResult]):
        self._queryResults = value


class VStorageObjectQuerySpec(vmodl.DynamicData):
    @property
    def queryField(self) -> str: ...
    @queryField.setter
    def queryField(self, value: str):
        self._queryField = value
    @property
    def queryOperator(self) -> str: ...
    @queryOperator.setter
    def queryOperator(self, value: str):
        self._queryOperator = value
    @property
    def queryValue(self) -> List[str]: ...
    @queryValue.setter
    def queryValue(self, value: List[str]):
        self._queryValue = value


    class QueryFieldEnum(Enum):
        id = "id"
        name = "name"
        capacity = "capacity"
        createTime = "createTime"
        backingObjectId = "backingObjectId"
        datastoreMoId = "datastoreMoId"
        metadataKey = "metadataKey"
        metadataValue = "metadataValue"


    class QueryOperatorEnum(Enum):
        equals = "equals"
        notEquals = "notEquals"
        lessThan = "lessThan"
        greaterThan = "greaterThan"
        lessThanOrEqual = "lessThanOrEqual"
        greaterThanOrEqual = "greaterThanOrEqual"
        contains = "contains"
        startsWith = "startsWith"
        endsWith = "endsWith"


class VStorageObjectResult(vmodl.DynamicData):
    @property
    def id(self) -> vim.vslm.ID: ...
    @id.setter
    def id(self, value: vim.vslm.ID):
        self._id = value
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str):
        self._name = value
    @property
    def capacityInMB(self) -> long: ...
    @capacityInMB.setter
    def capacityInMB(self, value: long):
        self._capacityInMB = value
    @property
    def createTime(self) -> datetime: ...
    @createTime.setter
    def createTime(self, value: datetime):
        self._createTime = value
    @property
    def datastoreUrl(self) -> str: ...
    @datastoreUrl.setter
    def datastoreUrl(self, value: str):
        self._datastoreUrl = value
    @property
    def diskPath(self) -> str: ...
    @diskPath.setter
    def diskPath(self, value: str):
        self._diskPath = value
    @property
    def usedCapacityInMB(self) -> long: ...
    @usedCapacityInMB.setter
    def usedCapacityInMB(self, value: long):
        self._usedCapacityInMB = value
    @property
    def backingObjectId(self) -> vim.vslm.ID: ...
    @backingObjectId.setter
    def backingObjectId(self, value: vim.vslm.ID):
        self._backingObjectId = value
    @property
    def snapshotInfo(self) -> List[VStorageObjectSnapshotResult]: ...
    @snapshotInfo.setter
    def snapshotInfo(self, value: List[VStorageObjectSnapshotResult]):
        self._snapshotInfo = value
    @property
    def metadata(self) -> List[vim.KeyValue]: ...
    @metadata.setter
    def metadata(self, value: List[vim.KeyValue]):
        self._metadata = value
    @property
    def error(self) -> vmodl.MethodFault: ...
    @error.setter
    def error(self, value: vmodl.MethodFault):
        self._error = value


class VStorageObjectSnapshotResult(vmodl.DynamicData):
    @property
    def backingObjectId(self) -> vim.vslm.ID: ...
    @backingObjectId.setter
    def backingObjectId(self, value: vim.vslm.ID):
        self._backingObjectId = value
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, value: str):
        self._description = value
    @property
    def snapshotId(self) -> vim.vslm.ID: ...
    @snapshotId.setter
    def snapshotId(self, value: vim.vslm.ID):
        self._snapshotId = value
    @property
    def diskPath(self) -> str: ...
    @diskPath.setter
    def diskPath(self, value: str):
        self._diskPath = value


from pyVmomi import vmodl
from pyVmomi.VmomiSupport import ManagedObject, PropertyPath


class HostCommunication(vmodl.RuntimeFault): ...


class HostNotConnected(HostCommunication): ...


class HostNotReachable(HostCommunication): ...


class InvalidArgument(vmodl.RuntimeFault):
    @property
    def invalidProperty(self) -> PropertyPath: ...
    @invalidProperty.setter
    def invalidProperty(self, value: PropertyPath):
        self._invalidProperty = value


class InvalidRequest(vmodl.RuntimeFault): ...


class InvalidType(InvalidRequest):
    @property
    def argument(self) -> PropertyPath: ...
    @argument.setter
    def argument(self, value: PropertyPath):
        self._argument = value


class ManagedObjectNotFound(vmodl.RuntimeFault):
    @property
    def obj(self) -> ManagedObject: ...
    @obj.setter
    def obj(self, value: ManagedObject):
        self._obj = value


class MethodNotFound(InvalidRequest):
    @property
    def receiver(self) -> ManagedObject: ...
    @receiver.setter
    def receiver(self, value: ManagedObject):
        self._receiver = value
    @property
    def method(self) -> str: ...
    @method.setter
    def method(self, value: str):
        self._method = value


class NotEnoughLicenses(vmodl.RuntimeFault): ...


class NotImplemented(vmodl.RuntimeFault): ...


class NotSupported(vmodl.RuntimeFault): ...


class RequestCanceled(vmodl.RuntimeFault): ...


class SecurityError(vmodl.RuntimeFault): ...


class SystemError(vmodl.RuntimeFault):
    @property
    def reason(self) -> str: ...
    @reason.setter
    def reason(self, value: str):
        self._reason = value


class UnexpectedFault(vmodl.RuntimeFault):
    @property
    def faultName(self) -> type: ...
    @faultName.setter
    def faultName(self, value: type):
        self._faultName = value
    @property
    def fault(self) -> vmodl.MethodFault: ...
    @fault.setter
    def fault(self, value: vmodl.MethodFault):
        self._fault = value


from typing import List, Literal
from enum import Enum
from pyVmomi import vim, vmodl
from pyVmomi.VmomiSupport import double, long


class ApplyRecommendationResult(vmodl.DynamicData):
    @property
    def vm(self) -> vim.VirtualMachine: ...
    @vm.setter
    def vm(self, value: vim.VirtualMachine):
        self._vm = value


class AutomationConfig(vmodl.DynamicData):
    @property
    def spaceLoadBalanceAutomationMode(self) -> str: ...
    @spaceLoadBalanceAutomationMode.setter
    def spaceLoadBalanceAutomationMode(self, value: str):
        self._spaceLoadBalanceAutomationMode = value
    @property
    def ioLoadBalanceAutomationMode(self) -> str: ...
    @ioLoadBalanceAutomationMode.setter
    def ioLoadBalanceAutomationMode(self, value: str):
        self._ioLoadBalanceAutomationMode = value
    @property
    def ruleEnforcementAutomationMode(self) -> str: ...
    @ruleEnforcementAutomationMode.setter
    def ruleEnforcementAutomationMode(self, value: str):
        self._ruleEnforcementAutomationMode = value
    @property
    def policyEnforcementAutomationMode(self) -> str: ...
    @policyEnforcementAutomationMode.setter
    def policyEnforcementAutomationMode(self, value: str):
        self._policyEnforcementAutomationMode = value
    @property
    def vmEvacuationAutomationMode(self) -> str: ...
    @vmEvacuationAutomationMode.setter
    def vmEvacuationAutomationMode(self, value: str):
        self._vmEvacuationAutomationMode = value


class ConfigInfo(vmodl.DynamicData):
    @property
    def podConfig(self) -> PodConfigInfo: ...
    @podConfig.setter
    def podConfig(self, value: PodConfigInfo):
        self._podConfig = value
    @property
    def vmConfig(self) -> List[VmConfigInfo]: ...
    @vmConfig.setter
    def vmConfig(self, value: List[VmConfigInfo]):
        self._vmConfig = value


class ConfigSpec(vmodl.DynamicData):
    @property
    def podConfigSpec(self) -> PodConfigSpec: ...
    @podConfigSpec.setter
    def podConfigSpec(self, value: PodConfigSpec):
        self._podConfigSpec = value
    @property
    def vmConfigSpec(self) -> List[VmConfigSpec]: ...
    @vmConfigSpec.setter
    def vmConfigSpec(self, value: List[VmConfigSpec]):
        self._vmConfigSpec = value


class HbrDiskMigrationAction(vim.cluster.Action):
    @property
    def collectionId(self) -> str: ...
    @collectionId.setter
    def collectionId(self, value: str):
        self._collectionId = value
    @property
    def collectionName(self) -> str: ...
    @collectionName.setter
    def collectionName(self, value: str):
        self._collectionName = value
    @property
    def diskIds(self) -> List[str]: ...
    @diskIds.setter
    def diskIds(self, value: List[str]):
        self._diskIds = value
    @property
    def source(self) -> vim.Datastore: ...
    @source.setter
    def source(self, value: vim.Datastore):
        self._source = value
    @property
    def destination(self) -> vim.Datastore: ...
    @destination.setter
    def destination(self, value: vim.Datastore):
        self._destination = value
    @property
    def sizeTransferred(self) -> long: ...
    @sizeTransferred.setter
    def sizeTransferred(self, value: long):
        self._sizeTransferred = value
    @property
    def spaceUtilSrcBefore(self) -> float: ...
    @spaceUtilSrcBefore.setter
    def spaceUtilSrcBefore(self, value: float):
        self._spaceUtilSrcBefore = value
    @property
    def spaceUtilDstBefore(self) -> float: ...
    @spaceUtilDstBefore.setter
    def spaceUtilDstBefore(self, value: float):
        self._spaceUtilDstBefore = value
    @property
    def spaceUtilSrcAfter(self) -> float: ...
    @spaceUtilSrcAfter.setter
    def spaceUtilSrcAfter(self, value: float):
        self._spaceUtilSrcAfter = value
    @property
    def spaceUtilDstAfter(self) -> float: ...
    @spaceUtilDstAfter.setter
    def spaceUtilDstAfter(self, value: float):
        self._spaceUtilDstAfter = value
    @property
    def ioLatencySrcBefore(self) -> float: ...
    @ioLatencySrcBefore.setter
    def ioLatencySrcBefore(self, value: float):
        self._ioLatencySrcBefore = value
    @property
    def ioLatencyDstBefore(self) -> float: ...
    @ioLatencyDstBefore.setter
    def ioLatencyDstBefore(self, value: float):
        self._ioLatencyDstBefore = value


class IoLoadBalanceConfig(vmodl.DynamicData):
    @property
    def reservablePercentThreshold(self) -> int: ...
    @reservablePercentThreshold.setter
    def reservablePercentThreshold(self, value: int):
        self._reservablePercentThreshold = value
    @property
    def reservableIopsThreshold(self) -> int: ...
    @reservableIopsThreshold.setter
    def reservableIopsThreshold(self, value: int):
        self._reservableIopsThreshold = value
    @property
    def reservableThresholdMode(self) -> str: ...
    @reservableThresholdMode.setter
    def reservableThresholdMode(self, value: str):
        self._reservableThresholdMode = value
    @property
    def ioLatencyThreshold(self) -> int: ...
    @ioLatencyThreshold.setter
    def ioLatencyThreshold(self, value: int):
        self._ioLatencyThreshold = value
    @property
    def ioLoadImbalanceThreshold(self) -> int: ...
    @ioLoadImbalanceThreshold.setter
    def ioLoadImbalanceThreshold(self, value: int):
        self._ioLoadImbalanceThreshold = value


class OptionSpec(vim.option.ArrayUpdateSpec):
    @property
    def option(self) -> vim.option.OptionValue: ...
    @option.setter
    def option(self, value: vim.option.OptionValue):
        self._option = value


class PlacementAffinityRule(vmodl.DynamicData):
    @property
    def ruleType(self) -> str: ...
    @ruleType.setter
    def ruleType(self, value: str):
        self._ruleType = value
    @property
    def ruleScope(self) -> str: ...
    @ruleScope.setter
    def ruleScope(self, value: str):
        self._ruleScope = value
    @property
    def vms(self) -> List[vim.VirtualMachine]: ...
    @vms.setter
    def vms(self, value: List[vim.VirtualMachine]):
        self._vms = value
    @property
    def keys(self) -> List[str]: ...
    @keys.setter
    def keys(self, value: List[str]):
        self._keys = value


    class RuleScope(Enum):
        cluster = "cluster"
        host = "host"
        storagePod = "storagePod"
        datastore = "datastore"


    class RuleType(Enum):
        affinity = "affinity"
        antiAffinity = "antiAffinity"
        softAffinity = "softAffinity"
        softAntiAffinity = "softAntiAffinity"


class PlacementRankResult(vmodl.DynamicData):
    @property
    def key(self) -> str: ...
    @key.setter
    def key(self, value: str):
        self._key = value
    @property
    def candidate(self) -> vim.ClusterComputeResource: ...
    @candidate.setter
    def candidate(self, value: vim.ClusterComputeResource):
        self._candidate = value
    @property
    def reservedSpaceMB(self) -> long: ...
    @reservedSpaceMB.setter
    def reservedSpaceMB(self, value: long):
        self._reservedSpaceMB = value
    @property
    def usedSpaceMB(self) -> long: ...
    @usedSpaceMB.setter
    def usedSpaceMB(self, value: long):
        self._usedSpaceMB = value
    @property
    def totalSpaceMB(self) -> long: ...
    @totalSpaceMB.setter
    def totalSpaceMB(self, value: long):
        self._totalSpaceMB = value
    @property
    def utilization(self) -> double: ...
    @utilization.setter
    def utilization(self, value: double):
        self._utilization = value
    @property
    def faults(self) -> List[vmodl.MethodFault]: ...
    @faults.setter
    def faults(self, value: List[vmodl.MethodFault]):
        self._faults = value


class PlacementRankSpec(vmodl.DynamicData):
    @property
    def specs(self) -> List[vim.cluster.PlacementSpec]: ...
    @specs.setter
    def specs(self, value: List[vim.cluster.PlacementSpec]):
        self._specs = value
    @property
    def clusters(self) -> List[vim.ClusterComputeResource]: ...
    @clusters.setter
    def clusters(self, value: List[vim.ClusterComputeResource]):
        self._clusters = value
    @property
    def rules(self) -> List[PlacementAffinityRule]: ...
    @rules.setter
    def rules(self, value: List[PlacementAffinityRule]):
        self._rules = value
    @property
    def placementRankByVm(self) -> List[PlacementRankVmSpec]: ...
    @placementRankByVm.setter
    def placementRankByVm(self, value: List[PlacementRankVmSpec]):
        self._placementRankByVm = value


class PlacementRankVmSpec(vmodl.DynamicData):
    @property
    def vmPlacementSpec(self) -> vim.cluster.PlacementSpec: ...
    @vmPlacementSpec.setter
    def vmPlacementSpec(self, value: vim.cluster.PlacementSpec):
        self._vmPlacementSpec = value
    @property
    def vmClusters(self) -> List[vim.ClusterComputeResource]: ...
    @vmClusters.setter
    def vmClusters(self, value: List[vim.ClusterComputeResource]):
        self._vmClusters = value


class PodConfigInfo(vmodl.DynamicData):
    @property
    def enabled(self) -> bool: ...
    @enabled.setter
    def enabled(self, value: bool):
        self._enabled = value
    @property
    def ioLoadBalanceEnabled(self) -> bool: ...
    @ioLoadBalanceEnabled.setter
    def ioLoadBalanceEnabled(self, value: bool):
        self._ioLoadBalanceEnabled = value
    @property
    def defaultVmBehavior(self) -> str: ...
    @defaultVmBehavior.setter
    def defaultVmBehavior(self, value: str):
        self._defaultVmBehavior = value
    @property
    def loadBalanceInterval(self) -> int: ...
    @loadBalanceInterval.setter
    def loadBalanceInterval(self, value: int):
        self._loadBalanceInterval = value
    @property
    def defaultIntraVmAffinity(self) -> bool: ...
    @defaultIntraVmAffinity.setter
    def defaultIntraVmAffinity(self, value: bool):
        self._defaultIntraVmAffinity = value
    @property
    def spaceLoadBalanceConfig(self) -> SpaceLoadBalanceConfig: ...
    @spaceLoadBalanceConfig.setter
    def spaceLoadBalanceConfig(self, value: SpaceLoadBalanceConfig):
        self._spaceLoadBalanceConfig = value
    @property
    def ioLoadBalanceConfig(self) -> IoLoadBalanceConfig: ...
    @ioLoadBalanceConfig.setter
    def ioLoadBalanceConfig(self, value: IoLoadBalanceConfig):
        self._ioLoadBalanceConfig = value
    @property
    def automationOverrides(self) -> AutomationConfig: ...
    @automationOverrides.setter
    def automationOverrides(self, value: AutomationConfig):
        self._automationOverrides = value
    @property
    def rule(self) -> List[vim.cluster.RuleInfo]: ...
    @rule.setter
    def rule(self, value: List[vim.cluster.RuleInfo]):
        self._rule = value
    @property
    def option(self) -> List[vim.option.OptionValue]: ...
    @option.setter
    def option(self, value: List[vim.option.OptionValue]):
        self._option = value


    class Behavior(Enum):
        manual = "manual"
        automated = "automated"


class PodConfigSpec(vmodl.DynamicData):
    @property
    def enabled(self) -> bool: ...
    @enabled.setter
    def enabled(self, value: bool):
        self._enabled = value
    @property
    def ioLoadBalanceEnabled(self) -> bool: ...
    @ioLoadBalanceEnabled.setter
    def ioLoadBalanceEnabled(self, value: bool):
        self._ioLoadBalanceEnabled = value
    @property
    def defaultVmBehavior(self) -> str: ...
    @defaultVmBehavior.setter
    def defaultVmBehavior(self, value: str):
        self._defaultVmBehavior = value
    @property
    def loadBalanceInterval(self) -> int: ...
    @loadBalanceInterval.setter
    def loadBalanceInterval(self, value: int):
        self._loadBalanceInterval = value
    @property
    def defaultIntraVmAffinity(self) -> bool: ...
    @defaultIntraVmAffinity.setter
    def defaultIntraVmAffinity(self, value: bool):
        self._defaultIntraVmAffinity = value
    @property
    def spaceLoadBalanceConfig(self) -> SpaceLoadBalanceConfig: ...
    @spaceLoadBalanceConfig.setter
    def spaceLoadBalanceConfig(self, value: SpaceLoadBalanceConfig):
        self._spaceLoadBalanceConfig = value
    @property
    def ioLoadBalanceConfig(self) -> IoLoadBalanceConfig: ...
    @ioLoadBalanceConfig.setter
    def ioLoadBalanceConfig(self, value: IoLoadBalanceConfig):
        self._ioLoadBalanceConfig = value
    @property
    def automationOverrides(self) -> AutomationConfig: ...
    @automationOverrides.setter
    def automationOverrides(self, value: AutomationConfig):
        self._automationOverrides = value
    @property
    def rule(self) -> List[vim.cluster.RuleSpec]: ...
    @rule.setter
    def rule(self, value: List[vim.cluster.RuleSpec]):
        self._rule = value
    @property
    def option(self) -> List[OptionSpec]: ...
    @option.setter
    def option(self, value: List[OptionSpec]):
        self._option = value


class PodSelectionSpec(vmodl.DynamicData):
    @property
    def initialVmConfig(self) -> List[PodSelectionSpec.VmPodConfig]: ...
    @initialVmConfig.setter
    def initialVmConfig(self, value: List[PodSelectionSpec.VmPodConfig]):
        self._initialVmConfig = value
    @property
    def storagePod(self) -> vim.StoragePod: ...
    @storagePod.setter
    def storagePod(self, value: vim.StoragePod):
        self._storagePod = value


    class DiskLocator(vmodl.DynamicData):
        @property
        def diskId(self) -> int: ...
        @diskId.setter
        def diskId(self, value: int):
            self._diskId = value
        @property
        def diskMoveType(self) -> str: ...
        @diskMoveType.setter
        def diskMoveType(self, value: str):
            self._diskMoveType = value
        @property
        def diskBackingInfo(self) -> vim.vm.device.VirtualDevice.BackingInfo: ...
        @diskBackingInfo.setter
        def diskBackingInfo(self, value: vim.vm.device.VirtualDevice.BackingInfo):
            self._diskBackingInfo = value
        @property
        def profile(self) -> List[vim.vm.ProfileSpec]: ...
        @profile.setter
        def profile(self, value: List[vim.vm.ProfileSpec]):
            self._profile = value


    class VmPodConfig(vmodl.DynamicData):
        @property
        def storagePod(self) -> vim.StoragePod: ...
        @storagePod.setter
        def storagePod(self, value: vim.StoragePod):
            self._storagePod = value
        @property
        def disk(self) -> List[PodSelectionSpec.DiskLocator]: ...
        @disk.setter
        def disk(self, value: List[PodSelectionSpec.DiskLocator]):
            self._disk = value
        @property
        def vmConfig(self) -> VmConfigInfo: ...
        @vmConfig.setter
        def vmConfig(self, value: VmConfigInfo):
            self._vmConfig = value
        @property
        def interVmRule(self) -> List[vim.cluster.RuleInfo]: ...
        @interVmRule.setter
        def interVmRule(self, value: List[vim.cluster.RuleInfo]):
            self._interVmRule = value


class SpaceLoadBalanceConfig(vmodl.DynamicData):
    @property
    def spaceThresholdMode(self) -> str: ...
    @spaceThresholdMode.setter
    def spaceThresholdMode(self, value: str):
        self._spaceThresholdMode = value
    @property
    def spaceUtilizationThreshold(self) -> int: ...
    @spaceUtilizationThreshold.setter
    def spaceUtilizationThreshold(self, value: int):
        self._spaceUtilizationThreshold = value
    @property
    def freeSpaceThresholdGB(self) -> int: ...
    @freeSpaceThresholdGB.setter
    def freeSpaceThresholdGB(self, value: int):
        self._freeSpaceThresholdGB = value
    @property
    def minSpaceUtilizationDifference(self) -> int: ...
    @minSpaceUtilizationDifference.setter
    def minSpaceUtilizationDifference(self, value: int):
        self._minSpaceUtilizationDifference = value


    class SpaceThresholdMode(Enum):
        utilization = "utilization"
        freeSpace = "freeSpace"


class StorageMigrationAction(vim.cluster.Action):
    @property
    def vm(self) -> vim.VirtualMachine: ...
    @vm.setter
    def vm(self, value: vim.VirtualMachine):
        self._vm = value
    @property
    def relocateSpec(self) -> vim.vm.RelocateSpec: ...
    @relocateSpec.setter
    def relocateSpec(self, value: vim.vm.RelocateSpec):
        self._relocateSpec = value
    @property
    def source(self) -> vim.Datastore: ...
    @source.setter
    def source(self, value: vim.Datastore):
        self._source = value
    @property
    def destination(self) -> vim.Datastore: ...
    @destination.setter
    def destination(self, value: vim.Datastore):
        self._destination = value
    @property
    def sizeTransferred(self) -> long: ...
    @sizeTransferred.setter
    def sizeTransferred(self, value: long):
        self._sizeTransferred = value
    @property
    def spaceUtilSrcBefore(self) -> float: ...
    @spaceUtilSrcBefore.setter
    def spaceUtilSrcBefore(self, value: float):
        self._spaceUtilSrcBefore = value
    @property
    def spaceUtilDstBefore(self) -> float: ...
    @spaceUtilDstBefore.setter
    def spaceUtilDstBefore(self, value: float):
        self._spaceUtilDstBefore = value
    @property
    def spaceUtilSrcAfter(self) -> float: ...
    @spaceUtilSrcAfter.setter
    def spaceUtilSrcAfter(self, value: float):
        self._spaceUtilSrcAfter = value
    @property
    def spaceUtilDstAfter(self) -> float: ...
    @spaceUtilDstAfter.setter
    def spaceUtilDstAfter(self, value: float):
        self._spaceUtilDstAfter = value
    @property
    def ioLatencySrcBefore(self) -> float: ...
    @ioLatencySrcBefore.setter
    def ioLatencySrcBefore(self, value: float):
        self._ioLatencySrcBefore = value
    @property
    def ioLatencyDstBefore(self) -> float: ...
    @ioLatencyDstBefore.setter
    def ioLatencyDstBefore(self, value: float):
        self._ioLatencyDstBefore = value


class StoragePlacementAction(vim.cluster.Action):
    @property
    def vm(self) -> vim.VirtualMachine: ...
    @vm.setter
    def vm(self, value: vim.VirtualMachine):
        self._vm = value
    @property
    def relocateSpec(self) -> vim.vm.RelocateSpec: ...
    @relocateSpec.setter
    def relocateSpec(self, value: vim.vm.RelocateSpec):
        self._relocateSpec = value
    @property
    def destination(self) -> vim.Datastore: ...
    @destination.setter
    def destination(self, value: vim.Datastore):
        self._destination = value
    @property
    def spaceUtilBefore(self) -> float: ...
    @spaceUtilBefore.setter
    def spaceUtilBefore(self, value: float):
        self._spaceUtilBefore = value
    @property
    def spaceDemandBefore(self) -> float: ...
    @spaceDemandBefore.setter
    def spaceDemandBefore(self, value: float):
        self._spaceDemandBefore = value
    @property
    def spaceUtilAfter(self) -> float: ...
    @spaceUtilAfter.setter
    def spaceUtilAfter(self, value: float):
        self._spaceUtilAfter = value
    @property
    def spaceDemandAfter(self) -> float: ...
    @spaceDemandAfter.setter
    def spaceDemandAfter(self, value: float):
        self._spaceDemandAfter = value
    @property
    def ioLatencyBefore(self) -> float: ...
    @ioLatencyBefore.setter
    def ioLatencyBefore(self, value: float):
        self._ioLatencyBefore = value


class StoragePlacementResult(vmodl.DynamicData):
    @property
    def recommendations(self) -> List[vim.cluster.Recommendation]: ...
    @recommendations.setter
    def recommendations(self, value: List[vim.cluster.Recommendation]):
        self._recommendations = value
    @property
    def drsFault(self) -> vim.cluster.DrsFaults: ...
    @drsFault.setter
    def drsFault(self, value: vim.cluster.DrsFaults):
        self._drsFault = value
    @property
    def task(self) -> vim.Task: ...
    @task.setter
    def task(self, value: vim.Task):
        self._task = value


class StoragePlacementSpec(vmodl.DynamicData):
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, value: str):
        self._type = value
    @property
    def priority(self) -> vim.VirtualMachine.MovePriority | Literal['lowPriority', 'highPriority', 'defaultPriority']: ...
    @priority.setter
    def priority(self, value: vim.VirtualMachine.MovePriority | Literal['lowPriority', 'highPriority', 'defaultPriority']):
        self._priority = value
    @property
    def vm(self) -> vim.VirtualMachine: ...
    @vm.setter
    def vm(self, value: vim.VirtualMachine):
        self._vm = value
    @property
    def podSelectionSpec(self) -> PodSelectionSpec: ...
    @podSelectionSpec.setter
    def podSelectionSpec(self, value: PodSelectionSpec):
        self._podSelectionSpec = value
    @property
    def cloneSpec(self) -> vim.vm.CloneSpec: ...
    @cloneSpec.setter
    def cloneSpec(self, value: vim.vm.CloneSpec):
        self._cloneSpec = value
    @property
    def cloneName(self) -> str: ...
    @cloneName.setter
    def cloneName(self, value: str):
        self._cloneName = value
    @property
    def configSpec(self) -> vim.vm.ConfigSpec: ...
    @configSpec.setter
    def configSpec(self, value: vim.vm.ConfigSpec):
        self._configSpec = value
    @property
    def relocateSpec(self) -> vim.vm.RelocateSpec: ...
    @relocateSpec.setter
    def relocateSpec(self, value: vim.vm.RelocateSpec):
        self._relocateSpec = value
    @property
    def resourcePool(self) -> vim.ResourcePool: ...
    @resourcePool.setter
    def resourcePool(self, value: vim.ResourcePool):
        self._resourcePool = value
    @property
    def host(self) -> vim.HostSystem: ...
    @host.setter
    def host(self, value: vim.HostSystem):
        self._host = value
    @property
    def folder(self) -> vim.Folder: ...
    @folder.setter
    def folder(self, value: vim.Folder):
        self._folder = value
    @property
    def disallowPrerequisiteMoves(self) -> bool: ...
    @disallowPrerequisiteMoves.setter
    def disallowPrerequisiteMoves(self, value: bool):
        self._disallowPrerequisiteMoves = value
    @property
    def resourceLeaseDurationSec(self) -> int: ...
    @resourceLeaseDurationSec.setter
    def resourceLeaseDurationSec(self, value: int):
        self._resourceLeaseDurationSec = value


    class PlacementType(Enum):
        create = "create"
        reconfigure = "reconfigure"
        relocate = "relocate"
        clone = "clone"


class VirtualDiskAntiAffinityRuleSpec(vim.cluster.RuleInfo):
    @property
    def diskId(self) -> List[int]: ...
    @diskId.setter
    def diskId(self, value: List[int]):
        self._diskId = value


class VirtualDiskRuleSpec(vim.cluster.RuleInfo):
    @property
    def diskRuleType(self) -> str: ...
    @diskRuleType.setter
    def diskRuleType(self, value: str):
        self._diskRuleType = value
    @property
    def diskId(self) -> List[int]: ...
    @diskId.setter
    def diskId(self, value: List[int]):
        self._diskId = value


class VmConfigInfo(vmodl.DynamicData):
    @property
    def vm(self) -> vim.VirtualMachine: ...
    @vm.setter
    def vm(self, value: vim.VirtualMachine):
        self._vm = value
    @property
    def enabled(self) -> bool: ...
    @enabled.setter
    def enabled(self, value: bool):
        self._enabled = value
    @property
    def behavior(self) -> str: ...
    @behavior.setter
    def behavior(self, value: str):
        self._behavior = value
    @property
    def intraVmAffinity(self) -> bool: ...
    @intraVmAffinity.setter
    def intraVmAffinity(self, value: bool):
        self._intraVmAffinity = value
    @property
    def intraVmAntiAffinity(self) -> VirtualDiskAntiAffinityRuleSpec: ...
    @intraVmAntiAffinity.setter
    def intraVmAntiAffinity(self, value: VirtualDiskAntiAffinityRuleSpec):
        self._intraVmAntiAffinity = value
    @property
    def virtualDiskRules(self) -> List[VirtualDiskRuleSpec]: ...
    @virtualDiskRules.setter
    def virtualDiskRules(self, value: List[VirtualDiskRuleSpec]):
        self._virtualDiskRules = value


class VmConfigSpec(vim.option.ArrayUpdateSpec):
    @property
    def info(self) -> VmConfigInfo: ...
    @info.setter
    def info(self, value: VmConfigInfo):
        self._info = value